from customer_services.interactor.get_customer.dto import GetCustomerRequest
from customer_services.model import Customer
from customer_services.service import GetCustomerService


class GetCustomerInteractor(object):
    def __init__(self, get_customer_service):
        # type: (GetCustomerService) -> None
        
        self.get_customer_service = get_customer_service

    def get_customer(self, request):
        # type: (GetCustomerRequest) -> Customer

        return self.get_customer_service.get_customer(request.phone)
