class OperatorGoals(object):
    def __init__(self, operator_id, amount_goal, amount_sold, average_ticket, item_goals, tickets):
        self.id = operator_id
        self.amount_goal = amount_goal
        self.amount_sold = amount_sold
        self.average_ticket = average_ticket
        self.item_goals = item_goals
        self.tickets = tickets
