class Config(object):

    def __init__(self):
        self.loader_path = None
        self.component_name = None
        self.required_services = None
        self.host = None
        self.port = None
        self.allowed_payment_methods = None
        self.available_sale_types = None
        self.ruptured_items_cache_time = None
        self.api_key_mapping = None
        self.nf_type = None
        self.cors_origin_hosts = None
        self.images_path = None
        self.totem_config_path = None
        self.up_sell_part_codes_path = None


class PosMap(object):

    def __init__(self, pos_id=None, api_key=None, abstract_pos_id=None, user_id=None, user_password=None):
        self.pos_id = pos_id
        self.api_key = api_key
        self.abstract_pos_id = abstract_pos_id
        self.user_id = user_id
        self.user_password = user_password
