# encoding: utf-8
from typing import (
    Any,
    Dict
)


class TefTender(object):
    def __init__(
        self,
        row_database,   # type: Any
        tender_detail   # type: Dict
    ):
        # type: (...) -> None

        self.tender_descr = row_database.get_entry("TenderDescr")
        self.timestamp = row_database.get_entry("Timestamp")
        self.tender_amount = row_database.get_entry("TenderAmount")
        self.tender_detail = tender_detail
