class TotalTender(object):
    def __init__(self):
        self.tender_type = 0  # type: int
        self.tender_name = u""  # type: unicode
        self.quantity = 0  # type: int
        self.value = 0.0  # type: float
