from ._CommonProcessor import _CommonProcessor


class DialogResponseProcessor(_CommonProcessor):
    def parse_data(self, data, event_type):
        return data

    def call_business(self, data):
        # type: (str) -> None
        processor = self.payment_router.get_payment_processor()
        processor.dialog_response(data)
