from salechallengereport.model import SaleChallengeItem


class SaleChallengeHeaderDto(object):
    def __init__(self, sale_challenge, operator_name):
        # type: (SaleChallengeItem, str) -> ()

        self.sale_challenge_name = sale_challenge.name
        self.operator_name = operator_name
        self.start_date = sale_challenge.start_date
        self.end_date = sale_challenge.end_date
