data = (
'jjyim',    # 0x00
'jjyib',    # 0x01
'jjyibs',    # 0x02
'jjyis',    # 0x03
'jjyiss',    # 0x04
'jjying',    # 0x05
'jjyij',    # 0x06
'jjyic',    # 0x07
'jjyik',    # 0x08
'jjyit',    # 0x09
'jjyip',    # 0x0a
'jjyih',    # 0x0b
'jji',    # 0x0c
'jjig',    # 0x0d
'jjigg',    # 0x0e
'jjigs',    # 0x0f
'jjin',    # 0x10
'jjinj',    # 0x11
'jjinh',    # 0x12
'jjid',    # 0x13
'jjil',    # 0x14
'jjilg',    # 0x15
'jjilm',    # 0x16
'jjilb',    # 0x17
'jjils',    # 0x18
'jjilt',    # 0x19
'jjilp',    # 0x1a
'jjilh',    # 0x1b
'jjim',    # 0x1c
'jjib',    # 0x1d
'jjibs',    # 0x1e
'jjis',    # 0x1f
'jjiss',    # 0x20
'jjing',    # 0x21
'jjij',    # 0x22
'jjic',    # 0x23
'jjik',    # 0x24
'jjit',    # 0x25
'jjip',    # 0x26
'jjih',    # 0x27
'ca',    # 0x28
'cag',    # 0x29
'cagg',    # 0x2a
'cags',    # 0x2b
'can',    # 0x2c
'canj',    # 0x2d
'canh',    # 0x2e
'cad',    # 0x2f
'cal',    # 0x30
'calg',    # 0x31
'calm',    # 0x32
'calb',    # 0x33
'cals',    # 0x34
'calt',    # 0x35
'calp',    # 0x36
'calh',    # 0x37
'cam',    # 0x38
'cab',    # 0x39
'cabs',    # 0x3a
'cas',    # 0x3b
'cass',    # 0x3c
'cang',    # 0x3d
'caj',    # 0x3e
'cac',    # 0x3f
'cak',    # 0x40
'cat',    # 0x41
'cap',    # 0x42
'cah',    # 0x43
'cae',    # 0x44
'caeg',    # 0x45
'caegg',    # 0x46
'caegs',    # 0x47
'caen',    # 0x48
'caenj',    # 0x49
'caenh',    # 0x4a
'caed',    # 0x4b
'cael',    # 0x4c
'caelg',    # 0x4d
'caelm',    # 0x4e
'caelb',    # 0x4f
'caels',    # 0x50
'caelt',    # 0x51
'caelp',    # 0x52
'caelh',    # 0x53
'caem',    # 0x54
'caeb',    # 0x55
'caebs',    # 0x56
'caes',    # 0x57
'caess',    # 0x58
'caeng',    # 0x59
'caej',    # 0x5a
'caec',    # 0x5b
'caek',    # 0x5c
'caet',    # 0x5d
'caep',    # 0x5e
'caeh',    # 0x5f
'cya',    # 0x60
'cyag',    # 0x61
'cyagg',    # 0x62
'cyags',    # 0x63
'cyan',    # 0x64
'cyanj',    # 0x65
'cyanh',    # 0x66
'cyad',    # 0x67
'cyal',    # 0x68
'cyalg',    # 0x69
'cyalm',    # 0x6a
'cyalb',    # 0x6b
'cyals',    # 0x6c
'cyalt',    # 0x6d
'cyalp',    # 0x6e
'cyalh',    # 0x6f
'cyam',    # 0x70
'cyab',    # 0x71
'cyabs',    # 0x72
'cyas',    # 0x73
'cyass',    # 0x74
'cyang',    # 0x75
'cyaj',    # 0x76
'cyac',    # 0x77
'cyak',    # 0x78
'cyat',    # 0x79
'cyap',    # 0x7a
'cyah',    # 0x7b
'cyae',    # 0x7c
'cyaeg',    # 0x7d
'cyaegg',    # 0x7e
'cyaegs',    # 0x7f
'cyaen',    # 0x80
'cyaenj',    # 0x81
'cyaenh',    # 0x82
'cyaed',    # 0x83
'cyael',    # 0x84
'cyaelg',    # 0x85
'cyaelm',    # 0x86
'cyaelb',    # 0x87
'cyaels',    # 0x88
'cyaelt',    # 0x89
'cyaelp',    # 0x8a
'cyaelh',    # 0x8b
'cyaem',    # 0x8c
'cyaeb',    # 0x8d
'cyaebs',    # 0x8e
'cyaes',    # 0x8f
'cyaess',    # 0x90
'cyaeng',    # 0x91
'cyaej',    # 0x92
'cyaec',    # 0x93
'cyaek',    # 0x94
'cyaet',    # 0x95
'cyaep',    # 0x96
'cyaeh',    # 0x97
'ceo',    # 0x98
'ceog',    # 0x99
'ceogg',    # 0x9a
'ceogs',    # 0x9b
'ceon',    # 0x9c
'ceonj',    # 0x9d
'ceonh',    # 0x9e
'ceod',    # 0x9f
'ceol',    # 0xa0
'ceolg',    # 0xa1
'ceolm',    # 0xa2
'ceolb',    # 0xa3
'ceols',    # 0xa4
'ceolt',    # 0xa5
'ceolp',    # 0xa6
'ceolh',    # 0xa7
'ceom',    # 0xa8
'ceob',    # 0xa9
'ceobs',    # 0xaa
'ceos',    # 0xab
'ceoss',    # 0xac
'ceong',    # 0xad
'ceoj',    # 0xae
'ceoc',    # 0xaf
'ceok',    # 0xb0
'ceot',    # 0xb1
'ceop',    # 0xb2
'ceoh',    # 0xb3
'ce',    # 0xb4
'ceg',    # 0xb5
'cegg',    # 0xb6
'cegs',    # 0xb7
'cen',    # 0xb8
'cenj',    # 0xb9
'cenh',    # 0xba
'ced',    # 0xbb
'cel',    # 0xbc
'celg',    # 0xbd
'celm',    # 0xbe
'celb',    # 0xbf
'cels',    # 0xc0
'celt',    # 0xc1
'celp',    # 0xc2
'celh',    # 0xc3
'cem',    # 0xc4
'ceb',    # 0xc5
'cebs',    # 0xc6
'ces',    # 0xc7
'cess',    # 0xc8
'ceng',    # 0xc9
'cej',    # 0xca
'cec',    # 0xcb
'cek',    # 0xcc
'cet',    # 0xcd
'cep',    # 0xce
'ceh',    # 0xcf
'cyeo',    # 0xd0
'cyeog',    # 0xd1
'cyeogg',    # 0xd2
'cyeogs',    # 0xd3
'cyeon',    # 0xd4
'cyeonj',    # 0xd5
'cyeonh',    # 0xd6
'cyeod',    # 0xd7
'cyeol',    # 0xd8
'cyeolg',    # 0xd9
'cyeolm',    # 0xda
'cyeolb',    # 0xdb
'cyeols',    # 0xdc
'cyeolt',    # 0xdd
'cyeolp',    # 0xde
'cyeolh',    # 0xdf
'cyeom',    # 0xe0
'cyeob',    # 0xe1
'cyeobs',    # 0xe2
'cyeos',    # 0xe3
'cyeoss',    # 0xe4
'cyeong',    # 0xe5
'cyeoj',    # 0xe6
'cyeoc',    # 0xe7
'cyeok',    # 0xe8
'cyeot',    # 0xe9
'cyeop',    # 0xea
'cyeoh',    # 0xeb
'cye',    # 0xec
'cyeg',    # 0xed
'cyegg',    # 0xee
'cyegs',    # 0xef
'cyen',    # 0xf0
'cyenj',    # 0xf1
'cyenh',    # 0xf2
'cyed',    # 0xf3
'cyel',    # 0xf4
'cyelg',    # 0xf5
'cyelm',    # 0xf6
'cyelb',    # 0xf7
'cyels',    # 0xf8
'cyelt',    # 0xf9
'cyelp',    # 0xfa
'cyelh',    # 0xfb
'cyem',    # 0xfc
'cyeb',    # 0xfd
'cyebs',    # 0xfe
'cyes',    # 0xff
)
