data = (
'gwan',    # 0x00
'gwanj',    # 0x01
'gwanh',    # 0x02
'gwad',    # 0x03
'gwal',    # 0x04
'gwalg',    # 0x05
'gwalm',    # 0x06
'gwalb',    # 0x07
'gwals',    # 0x08
'gwalt',    # 0x09
'gwalp',    # 0x0a
'gwalh',    # 0x0b
'gwam',    # 0x0c
'gwab',    # 0x0d
'gwabs',    # 0x0e
'gwas',    # 0x0f
'gwass',    # 0x10
'gwang',    # 0x11
'gwaj',    # 0x12
'gwac',    # 0x13
'gwak',    # 0x14
'gwat',    # 0x15
'gwap',    # 0x16
'gwah',    # 0x17
'gwae',    # 0x18
'gwaeg',    # 0x19
'gwaegg',    # 0x1a
'gwaegs',    # 0x1b
'gwaen',    # 0x1c
'gwaenj',    # 0x1d
'gwaenh',    # 0x1e
'gwaed',    # 0x1f
'gwael',    # 0x20
'gwaelg',    # 0x21
'gwaelm',    # 0x22
'gwaelb',    # 0x23
'gwaels',    # 0x24
'gwaelt',    # 0x25
'gwaelp',    # 0x26
'gwaelh',    # 0x27
'gwaem',    # 0x28
'gwaeb',    # 0x29
'gwaebs',    # 0x2a
'gwaes',    # 0x2b
'gwaess',    # 0x2c
'gwaeng',    # 0x2d
'gwaej',    # 0x2e
'gwaec',    # 0x2f
'gwaek',    # 0x30
'gwaet',    # 0x31
'gwaep',    # 0x32
'gwaeh',    # 0x33
'goe',    # 0x34
'goeg',    # 0x35
'goegg',    # 0x36
'goegs',    # 0x37
'goen',    # 0x38
'goenj',    # 0x39
'goenh',    # 0x3a
'goed',    # 0x3b
'goel',    # 0x3c
'goelg',    # 0x3d
'goelm',    # 0x3e
'goelb',    # 0x3f
'goels',    # 0x40
'goelt',    # 0x41
'goelp',    # 0x42
'goelh',    # 0x43
'goem',    # 0x44
'goeb',    # 0x45
'goebs',    # 0x46
'goes',    # 0x47
'goess',    # 0x48
'goeng',    # 0x49
'goej',    # 0x4a
'goec',    # 0x4b
'goek',    # 0x4c
'goet',    # 0x4d
'goep',    # 0x4e
'goeh',    # 0x4f
'gyo',    # 0x50
'gyog',    # 0x51
'gyogg',    # 0x52
'gyogs',    # 0x53
'gyon',    # 0x54
'gyonj',    # 0x55
'gyonh',    # 0x56
'gyod',    # 0x57
'gyol',    # 0x58
'gyolg',    # 0x59
'gyolm',    # 0x5a
'gyolb',    # 0x5b
'gyols',    # 0x5c
'gyolt',    # 0x5d
'gyolp',    # 0x5e
'gyolh',    # 0x5f
'gyom',    # 0x60
'gyob',    # 0x61
'gyobs',    # 0x62
'gyos',    # 0x63
'gyoss',    # 0x64
'gyong',    # 0x65
'gyoj',    # 0x66
'gyoc',    # 0x67
'gyok',    # 0x68
'gyot',    # 0x69
'gyop',    # 0x6a
'gyoh',    # 0x6b
'gu',    # 0x6c
'gug',    # 0x6d
'gugg',    # 0x6e
'gugs',    # 0x6f
'gun',    # 0x70
'gunj',    # 0x71
'gunh',    # 0x72
'gud',    # 0x73
'gul',    # 0x74
'gulg',    # 0x75
'gulm',    # 0x76
'gulb',    # 0x77
'guls',    # 0x78
'gult',    # 0x79
'gulp',    # 0x7a
'gulh',    # 0x7b
'gum',    # 0x7c
'gub',    # 0x7d
'gubs',    # 0x7e
'gus',    # 0x7f
'guss',    # 0x80
'gung',    # 0x81
'guj',    # 0x82
'guc',    # 0x83
'guk',    # 0x84
'gut',    # 0x85
'gup',    # 0x86
'guh',    # 0x87
'gweo',    # 0x88
'gweog',    # 0x89
'gweogg',    # 0x8a
'gweogs',    # 0x8b
'gweon',    # 0x8c
'gweonj',    # 0x8d
'gweonh',    # 0x8e
'gweod',    # 0x8f
'gweol',    # 0x90
'gweolg',    # 0x91
'gweolm',    # 0x92
'gweolb',    # 0x93
'gweols',    # 0x94
'gweolt',    # 0x95
'gweolp',    # 0x96
'gweolh',    # 0x97
'gweom',    # 0x98
'gweob',    # 0x99
'gweobs',    # 0x9a
'gweos',    # 0x9b
'gweoss',    # 0x9c
'gweong',    # 0x9d
'gweoj',    # 0x9e
'gweoc',    # 0x9f
'gweok',    # 0xa0
'gweot',    # 0xa1
'gweop',    # 0xa2
'gweoh',    # 0xa3
'gwe',    # 0xa4
'gweg',    # 0xa5
'gwegg',    # 0xa6
'gwegs',    # 0xa7
'gwen',    # 0xa8
'gwenj',    # 0xa9
'gwenh',    # 0xaa
'gwed',    # 0xab
'gwel',    # 0xac
'gwelg',    # 0xad
'gwelm',    # 0xae
'gwelb',    # 0xaf
'gwels',    # 0xb0
'gwelt',    # 0xb1
'gwelp',    # 0xb2
'gwelh',    # 0xb3
'gwem',    # 0xb4
'gweb',    # 0xb5
'gwebs',    # 0xb6
'gwes',    # 0xb7
'gwess',    # 0xb8
'gweng',    # 0xb9
'gwej',    # 0xba
'gwec',    # 0xbb
'gwek',    # 0xbc
'gwet',    # 0xbd
'gwep',    # 0xbe
'gweh',    # 0xbf
'gwi',    # 0xc0
'gwig',    # 0xc1
'gwigg',    # 0xc2
'gwigs',    # 0xc3
'gwin',    # 0xc4
'gwinj',    # 0xc5
'gwinh',    # 0xc6
'gwid',    # 0xc7
'gwil',    # 0xc8
'gwilg',    # 0xc9
'gwilm',    # 0xca
'gwilb',    # 0xcb
'gwils',    # 0xcc
'gwilt',    # 0xcd
'gwilp',    # 0xce
'gwilh',    # 0xcf
'gwim',    # 0xd0
'gwib',    # 0xd1
'gwibs',    # 0xd2
'gwis',    # 0xd3
'gwiss',    # 0xd4
'gwing',    # 0xd5
'gwij',    # 0xd6
'gwic',    # 0xd7
'gwik',    # 0xd8
'gwit',    # 0xd9
'gwip',    # 0xda
'gwih',    # 0xdb
'gyu',    # 0xdc
'gyug',    # 0xdd
'gyugg',    # 0xde
'gyugs',    # 0xdf
'gyun',    # 0xe0
'gyunj',    # 0xe1
'gyunh',    # 0xe2
'gyud',    # 0xe3
'gyul',    # 0xe4
'gyulg',    # 0xe5
'gyulm',    # 0xe6
'gyulb',    # 0xe7
'gyuls',    # 0xe8
'gyult',    # 0xe9
'gyulp',    # 0xea
'gyulh',    # 0xeb
'gyum',    # 0xec
'gyub',    # 0xed
'gyubs',    # 0xee
'gyus',    # 0xef
'gyuss',    # 0xf0
'gyung',    # 0xf1
'gyuj',    # 0xf2
'gyuc',    # 0xf3
'gyuk',    # 0xf4
'gyut',    # 0xf5
'gyup',    # 0xf6
'gyuh',    # 0xf7
'geu',    # 0xf8
'geug',    # 0xf9
'geugg',    # 0xfa
'geugs',    # 0xfb
'geun',    # 0xfc
'geunj',    # 0xfd
'geunh',    # 0xfe
'geud',    # 0xff
)
