class ProductSaleItem(object):

    def __init__(self, name, quantity, total_price):
        # type: (str, float, float) -> None

        self.name = name
        self.quantity = quantity
        self.total_price = total_price
