# encoding: utf-8

from ._BenefitsHeaderDto import BenefitsHeaderDto
from ._BenefitsBodyDto import BenefitsBodyDto


class BenefitsGeneratorDto(object):
    def __init__(self, header, body):
        # type: (BenefitsHeaderDto, BenefitsBodyDto) -> ()
        self.header = header
        self.body = body
