from httpmessageprocessorutil.automapped import auto_map
from model.order_detail import AdditionalProperty
from typing import List


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str},
           "to_add_properties": {"name": "toAddProperties", "type": List[AdditionalProperty]}})
class AddPropertiesRequest(object):
    def __init__(self, system_api_key, pos_id, to_add_properties):
        # type: (str, str, str) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
        self.to_add_properties = to_add_properties
