# encoding: utf-8
from datetime import datetime

from utils import get_store_id


class TefCancellationHeaderDto(object):

    def __init__(
        self,
        title,          # type: str
        start_date,     # type: datetime
        end_date        # type: datetime
    ):
        # type: (...) -> None

        self.title = title
        self.store_name = get_store_id()
        self.start_date = start_date
        self.end_date = end_date
