class GetStoreResponse(object):
    def __init__(
            self,
            status,     # type: bool
            menu_id     # type: str
    ):
        # type: (...) -> None

        self.isOnline = status
        self.menuId = menu_id
