from typing import List, Any

from salechallengereport.model import OperatorSaleItem, ProductSaleItem

from ._SaleChallengeHeaderDto import SaleChallengeHeaderDto


class SaleChallengeGeneratorDto(object):
    def __init__(self, header, items, columns):
        # type: (SaleChallengeHeaderDto, Any[List[OperatorSaleItem], List[ProductSaleItem]], int) -> ()
        self.header = header
        self.columns = columns
        self.items = items
