data = (
'syae',    # 0x00
'syaeg',    # 0x01
'syaegg',    # 0x02
'syaegs',    # 0x03
'syaen',    # 0x04
'syaenj',    # 0x05
'syaenh',    # 0x06
'syaed',    # 0x07
'syael',    # 0x08
'syaelg',    # 0x09
'syaelm',    # 0x0a
'syaelb',    # 0x0b
'syaels',    # 0x0c
'syaelt',    # 0x0d
'syaelp',    # 0x0e
'syaelh',    # 0x0f
'syaem',    # 0x10
'syaeb',    # 0x11
'syaebs',    # 0x12
'syaes',    # 0x13
'syaess',    # 0x14
'syaeng',    # 0x15
'syaej',    # 0x16
'syaec',    # 0x17
'syaek',    # 0x18
'syaet',    # 0x19
'syaep',    # 0x1a
'syaeh',    # 0x1b
'seo',    # 0x1c
'seog',    # 0x1d
'seogg',    # 0x1e
'seogs',    # 0x1f
'seon',    # 0x20
'seonj',    # 0x21
'seonh',    # 0x22
'seod',    # 0x23
'seol',    # 0x24
'seolg',    # 0x25
'seolm',    # 0x26
'seolb',    # 0x27
'seols',    # 0x28
'seolt',    # 0x29
'seolp',    # 0x2a
'seolh',    # 0x2b
'seom',    # 0x2c
'seob',    # 0x2d
'seobs',    # 0x2e
'seos',    # 0x2f
'seoss',    # 0x30
'seong',    # 0x31
'seoj',    # 0x32
'seoc',    # 0x33
'seok',    # 0x34
'seot',    # 0x35
'seop',    # 0x36
'seoh',    # 0x37
'se',    # 0x38
'seg',    # 0x39
'segg',    # 0x3a
'segs',    # 0x3b
'sen',    # 0x3c
'senj',    # 0x3d
'senh',    # 0x3e
'sed',    # 0x3f
'sel',    # 0x40
'selg',    # 0x41
'selm',    # 0x42
'selb',    # 0x43
'sels',    # 0x44
'selt',    # 0x45
'selp',    # 0x46
'selh',    # 0x47
'sem',    # 0x48
'seb',    # 0x49
'sebs',    # 0x4a
'ses',    # 0x4b
'sess',    # 0x4c
'seng',    # 0x4d
'sej',    # 0x4e
'sec',    # 0x4f
'sek',    # 0x50
'set',    # 0x51
'sep',    # 0x52
'seh',    # 0x53
'syeo',    # 0x54
'syeog',    # 0x55
'syeogg',    # 0x56
'syeogs',    # 0x57
'syeon',    # 0x58
'syeonj',    # 0x59
'syeonh',    # 0x5a
'syeod',    # 0x5b
'syeol',    # 0x5c
'syeolg',    # 0x5d
'syeolm',    # 0x5e
'syeolb',    # 0x5f
'syeols',    # 0x60
'syeolt',    # 0x61
'syeolp',    # 0x62
'syeolh',    # 0x63
'syeom',    # 0x64
'syeob',    # 0x65
'syeobs',    # 0x66
'syeos',    # 0x67
'syeoss',    # 0x68
'syeong',    # 0x69
'syeoj',    # 0x6a
'syeoc',    # 0x6b
'syeok',    # 0x6c
'syeot',    # 0x6d
'syeop',    # 0x6e
'syeoh',    # 0x6f
'sye',    # 0x70
'syeg',    # 0x71
'syegg',    # 0x72
'syegs',    # 0x73
'syen',    # 0x74
'syenj',    # 0x75
'syenh',    # 0x76
'syed',    # 0x77
'syel',    # 0x78
'syelg',    # 0x79
'syelm',    # 0x7a
'syelb',    # 0x7b
'syels',    # 0x7c
'syelt',    # 0x7d
'syelp',    # 0x7e
'syelh',    # 0x7f
'syem',    # 0x80
'syeb',    # 0x81
'syebs',    # 0x82
'syes',    # 0x83
'syess',    # 0x84
'syeng',    # 0x85
'syej',    # 0x86
'syec',    # 0x87
'syek',    # 0x88
'syet',    # 0x89
'syep',    # 0x8a
'syeh',    # 0x8b
'so',    # 0x8c
'sog',    # 0x8d
'sogg',    # 0x8e
'sogs',    # 0x8f
'son',    # 0x90
'sonj',    # 0x91
'sonh',    # 0x92
'sod',    # 0x93
'sol',    # 0x94
'solg',    # 0x95
'solm',    # 0x96
'solb',    # 0x97
'sols',    # 0x98
'solt',    # 0x99
'solp',    # 0x9a
'solh',    # 0x9b
'som',    # 0x9c
'sob',    # 0x9d
'sobs',    # 0x9e
'sos',    # 0x9f
'soss',    # 0xa0
'song',    # 0xa1
'soj',    # 0xa2
'soc',    # 0xa3
'sok',    # 0xa4
'sot',    # 0xa5
'sop',    # 0xa6
'soh',    # 0xa7
'swa',    # 0xa8
'swag',    # 0xa9
'swagg',    # 0xaa
'swags',    # 0xab
'swan',    # 0xac
'swanj',    # 0xad
'swanh',    # 0xae
'swad',    # 0xaf
'swal',    # 0xb0
'swalg',    # 0xb1
'swalm',    # 0xb2
'swalb',    # 0xb3
'swals',    # 0xb4
'swalt',    # 0xb5
'swalp',    # 0xb6
'swalh',    # 0xb7
'swam',    # 0xb8
'swab',    # 0xb9
'swabs',    # 0xba
'swas',    # 0xbb
'swass',    # 0xbc
'swang',    # 0xbd
'swaj',    # 0xbe
'swac',    # 0xbf
'swak',    # 0xc0
'swat',    # 0xc1
'swap',    # 0xc2
'swah',    # 0xc3
'swae',    # 0xc4
'swaeg',    # 0xc5
'swaegg',    # 0xc6
'swaegs',    # 0xc7
'swaen',    # 0xc8
'swaenj',    # 0xc9
'swaenh',    # 0xca
'swaed',    # 0xcb
'swael',    # 0xcc
'swaelg',    # 0xcd
'swaelm',    # 0xce
'swaelb',    # 0xcf
'swaels',    # 0xd0
'swaelt',    # 0xd1
'swaelp',    # 0xd2
'swaelh',    # 0xd3
'swaem',    # 0xd4
'swaeb',    # 0xd5
'swaebs',    # 0xd6
'swaes',    # 0xd7
'swaess',    # 0xd8
'swaeng',    # 0xd9
'swaej',    # 0xda
'swaec',    # 0xdb
'swaek',    # 0xdc
'swaet',    # 0xdd
'swaep',    # 0xde
'swaeh',    # 0xdf
'soe',    # 0xe0
'soeg',    # 0xe1
'soegg',    # 0xe2
'soegs',    # 0xe3
'soen',    # 0xe4
'soenj',    # 0xe5
'soenh',    # 0xe6
'soed',    # 0xe7
'soel',    # 0xe8
'soelg',    # 0xe9
'soelm',    # 0xea
'soelb',    # 0xeb
'soels',    # 0xec
'soelt',    # 0xed
'soelp',    # 0xee
'soelh',    # 0xef
'soem',    # 0xf0
'soeb',    # 0xf1
'soebs',    # 0xf2
'soes',    # 0xf3
'soess',    # 0xf4
'soeng',    # 0xf5
'soej',    # 0xf6
'soec',    # 0xf7
'soek',    # 0xf8
'soet',    # 0xf9
'soep',    # 0xfa
'soeh',    # 0xfb
'syo',    # 0xfc
'syog',    # 0xfd
'syogg',    # 0xfe
'syogs',    # 0xff
)
